


'''
	import fortunes.taps.season_3.super_hero_trend.win_rates.shares_trading as SHT_WR_shares_trading
	SHT_WR_shares_trading.calc (data)
'''

from fractions import Fraction
def calc (DF):
	win_rate = 1

	previous_amount = None;
	previous_signal = None;
	
	last_sell_signal = None
	last_buy_signal = None
	
	bought_at = None
	sold_at = None
	
	'''
		find BUY signal to SELL signal multiplier.
	'''	
	for index, row in DF.iterrows ():
		signal = row ['ST_BUY_SELL']
		
		#print (signal, previous_signal)
		
		if (signal == "SELL"):
			last_sell_signal = row ["close"]
		elif (signal == "BUY"):
			last_buy_signal = row ["close"]
		
		if (signal == "BUY" and previous_signal == "SELL"):
			#print ("BUY!", last_sell_signal)
			
			bought_at = Fraction (row ["close"])
			
			'''
			if (type (sold_at) == Fraction):
				multiplier = Fraction (row ["close"]) / Fraction (sold_at)	
				win_rate = win_rate * multiplier

				print ({
					"win rate": float (win_rate),
					"multiplier": float (multiplier),
					"span": [ float (bought_at), float (sold_at) ]
				})
			'''
			
		if (signal == "SELL" and previous_signal == "BUY"):
			#print ("SELL!", last_buy_signal, type (bought_at))
			
			sold_at = Fraction (row ["close"])
			
			if (type (bought_at) == Fraction):
				multiplier = Fraction (row ["close"]) / Fraction (bought_at)	
				win_rate = win_rate * multiplier

				print ({
					"win rate": float (win_rate),
					"multiplier": float (multiplier),
					"span": [ float (bought_at), float (sold_at) ]
				})
			
			
		previous_signal = signal;
		
	
	actual_change = float (
		Fraction (DF ["close"].iloc [-1]) / Fraction (DF ["close"].iloc [0])
	)	
		
	print ("win_rate:", float (win_rate))
	print ("actual change:", actual_change)
	
	
	return;



'''
	import fortunes.taps.season_3.physical_span_reversals as PSR
	PSR.calc ()
'''

'''
	https://tradingstrategy.ai/docs/_modules/pandas_ta/overlap/supertrend.html
'''

from fractions import Fraction

def calc (
	places, 
	spans = 14, 
	multiplier = 1.0
):
	'''
		This calculates spans between high, low, and close prices.
	'''
	import fortunes.taps.season_3.physical_span as PS_taps
	PS_taps.calc (
		places = places,
		label = "PS"
	)
	
	import rich
	rich.print_json (data = places)
	
	'''
	
	'''
	import fortunes.taps.season_3.physical_span_average as PSA_taps
	PSA = PSA_taps.calc (
		places = places,
		spans = spans
	)

	'''
		source = 
			((place_2 ["high"] + place_2 ["low"]) / 2)
			
		source = 
			open
			
		source = 
			close
			
		upper_band = candle['high'] + multiplier * atr
        lower_band = candle['low'] - multiplier * atr
	'''
	
	def upper_band_source (prices):
		return (prices ["high"] + prices ["low"]) / 2
		
	def lower_band_source (prices):
		return (prices ["high"] + prices ["low"]) / 2
	
	last_places_index = len (places) - 1;
	s = 1
	while (s <= last_places_index):
		place_1 = places [s - 1]
		place_2 = places [s]
		
		try:
			places [s] ["PSR UB"] = (
				upper_band_source (place_2) +
				(multiplier * place_2 ["PSA"])
			)
			places [s] ["PSR LB"] = (
				lower_band_source (place_2) -
				(multiplier * place_2 ["PSA"])
			)
		except Exception:
			pass;
			
		s += 1
		

	
	last_places_index = len (places) - 1;
	S = 1
	while (S <= last_places_index):
		place_1 = places [S - 1]
		place_0 = places [S]
		
		print ("PSR calc?")
		
		'''
			trend := 
				trend == -1 and close > lower_band_1 ? 
					1 : 
					trend == 1 and close < upper_band_1 ? -1 : trend
					
			if (
				place_1 ["PSR direction"] == -1 and 
				place_0 ['close'] > place_0 ["PSR LB"]
			):
				place_0 ["PSR direction"] = 1
				
			elif (
				place_1 ["PSR direction"] == 1 and 
				place_0 ['close'] < place_0 ["PSR UB"]
			):
				place_0 ["PSR direction"] = -1
				
			else:
				place_0 ["PSR direction"] = place_0 ["PSR direction"]
				
				
			if (place_0 ["PSR direction"] == -1):
				place_0 ["PSR"] = place_0 ["PSR LB"]
			else:
				place_0 ["PSR"] = place_0 ["PSR UB"]
		'''
		try:
			if ("PSR direction" not in place_1):
				place_0 ["PSR direction"] = 1;
		
			elif (
				place_1 ["PSR direction"] == -1 and 
				place_0 ['close'] > place_0 ["PSR LB"]
			):
				place_0 ["PSR direction"] = 1
				
			elif (
				place_1 ["PSR direction"] == 1 and 
				place_0 ['close'] < place_0 ["PSR UB"]
			):
				place_0 ["PSR direction"] = -1
			
			else:
				place_0 ["PSR direction"] == place_1 ["PSR direction"] 
			
				
			if (place_0 ["PSR direction"] == -1):
				place_0 ["PSR"] = place_0 ["PSR LB"]
			else:
				place_0 ["PSR"] = place_0 ["PSR UB"]
				
				
				
			'''
			place_2 ["PSR"] = 0
		
			if place_2 ['close'] > place_2 ["PSR UB"]:
				place_2 ["PSR"] = 1
			elif place_2 ['close'] < place_2 ["PSR LB"]:
				place_2 ["PSR"] = -1
			'''

		except Exception as E:
			print ("PSR Exception:", E)
		
			pass;
			
		S += 1

	'''
	for i, candle in enumerate (places):
		print ("i:", i)

		if (i == 0):
			continue;
	
		high = candle ['high']
		low = candle ['low']
		close = candle ['close']
		
		PS = candle ["PS"]
		PSA = candle ["PSA"]
	
		if i == 0:
			PSA = PS
			upper_band = high + multiplier * PSA
			lower_band = low - multiplier * PSA
		else:
			PSA = ((spans - 1) * PSA + PS) / spans
			upper_band = high + multiplier * PSA
			lower_band = low - multiplier * PSA

		PSA_1 = ((high + low) / 2) + (multiplier * PSA)


		if i == 0 or places [i - 1] ['close'] <= PSA:
			PSA = upper_band
		else:
			PSA = lower_band

	'''

	return places




'''
	balance sheet
		https://en.wikipedia.org/wiki/Balance_sheet
		
		https://www.nasdaq.com/market-activity/stocks/otly/sec-filings
		
		https://www.nasdaq.com/market-activity/stocks/otly/financials
		
		https://finance.yahoo.com/quote/OTLY/balance-sheet/
		https://stockanalysis.com/stocks/otly/financials/balance-sheet/?p=quarterly
		https://www.marketwatch.com/investing/stock/otly/financials/balance-sheet
		
		--
	
		equity
		
		---
		
		assets
		
		
		liabilities
		
		---
'''

'''


'''




'''
	import fortunes.taps.season_3.supertrend as supertrend
	DF ['supertrend'] = supertrend.calc (DF)
'''


import pandas as pd
import numpy as np

def calc (data, period=7, multiplier=3):
    high = data['high']
    low = data['low']
    close = data['close']

    atr = pd.Series(index=high.index)
    atr[0] = 0

    # Calculate Average True Range (ATR)
    for i in range(1, len(atr)):
        tr = max(high[i] - low[i], abs(high[i] - close[i - 1]), abs(low[i] - close[i - 1]))
        atr[i] = (atr[i - 1] * (period - 1) + tr) / period

    # Calculate SuperTrend
    upper_band = (high + low) / 2 + multiplier * atr
    lower_band = (high + low) / 2 - multiplier * atr

    supertrend = pd.Series(index=close.index)
    supertrend[0] = 0

    for i in range(1, len(supertrend)):
        if close[i - 1] <= supertrend[i - 1]:
            supertrend[i] = upper_band[i]
        else:
            supertrend[i] = lower_band[i]

    return supertrend






'''
	import fortunes.treasures.options.movement_calculator as option_movement_calculator
'''

'''
	priorities:
		for one option:
			inputs:
				option type:

				option ask price:	
				option strike price:	
				
				share price:
				
			outputs:
				
'''

'''
	There's 1 inflection point, the break even.
	
		at break even, the multiplier = 1
'''

'''
	ideally, calculate the slope of the line from:
		0 to the break even
		break even to infinity
		
	slope:
		x = share prices
		y = multipliers
		
		= dY / dX 
		
		= (y2 - y1)
		  ---
		  (x2 - x1)
'''

from fractions import Fraction

def calc (
	option_type = "",
	
	option_price = 0,
	strike_price = 0,
	
	share_price = 0
):
	
	#print ()

	expense = option_price
	break_even = option_price + strike_price;
	
	if (share_price <= strike_price):
		# out of the money
		revenue = 0
		
	else:
		# in the money
		revenue = Fraction (share_price) - Fraction (strike_price)

	
	#print ("break even share price:", float (break_even))
	#print ("expense:", float (expense))
	#print ("revenue:", float (revenue))
	
	balance = revenue - expense;
	
	"""
	print ()
	print (
		f'''One "${ 
			float (strike_price)
		}" "{ option_type }" option, bought for "${ 
			float (option_price) 
		}" while shares are "${ 
			float (share_price)
		}" each, yields a balance of "${
			float (balance)
		}".'''
	)
	print ()
	"""
	
	class Movement:
		balance = ""
		
	movement = Movement ()
	movement.balance = balance;
	movement.balance_multiplier = Fraction (revenue, expense);
	movement.break_even = break_even;

	return movement;




'''
	import fortunes.stats.aggregate_break_even as aggregate_break_even
	aggregate_break_even.calc ()
'''

'''
	priorities: {
		"expirations": [
			{
				"expiration": "2023-10-13",
				"aggregate break even": {
					"calls": {
						"ask":
						"bid":
						"last": 
					},
					"puts": {
						"ask":
						"bid":
						"last": 
					}
				}
			}
		]
	}

'''

from fractions import Fraction as F

prices = [ "ask", "bid", "last" ]

def calc (data):
	expirations = data ["expirations"]
	
	proceeds = {
		"expirations": []
	}
	
	for expiration in expirations:
		calls = expiration ["calls"]
		calls_strikes = expiration ["calls"] ["strikes"]
		
		puts = expiration ["puts"]
		puts_strikes = expiration ["puts"] ["strikes"]
		
		expiration_data = {
			"expiration": expiration ["expiration"],
			"aggregate break even": {
				"calls": {
					"ask": "",
					"bid": "",
					"last": ""
				},
				"puts": {
					"ask": "",
					"bid": "",
					"last": ""
				}
			},
			"aggregate multiplier": {
				"calls": {
					"ask": "",
					"bid": "",
					"last": ""
				},
				"puts": {
					"ask": "",
					"bid": "",
					"last": ""
				}
			}
		}


		contracts = [ "puts", "calls" ]
		for contract in contracts:
			sums = {
				"aggregate": {
					"ask": 0,
					"bid": 0,
					"last": 0
				},
				"confidence": {
					"ask": 0,
					"bid": 0,
					"last": 0
				},
				"multiplier numerator":{
					"ask": 0,
					"bid": 0,
					"last": 0
				}
			}

			for option in expiration [ contract ] ["strikes"]:
				for price in prices:
					try:
						strike = F (option ["strike"])
					
						if (contract == "calls"):
							bet = strike + F (option ["prices"] [ price ])
						else:
							if (strike < F (option ["prices"] [ price ])):
								raise Exception ("?")
						
							bet = strike - F (option ["prices"] [ price ])
						
						confidence = F (option ["prices"] [ price ]) * F (option ["open interest"])
						sum = bet * confidence
						
						sums ["aggregate"] [ price ] += sum
						sums ["confidence"] [ price ] += confidence
						
						if (contract == "calls"):
							sums ["multiplier numerator"] [ price ] += (bet / strike) * confidence
						else:
							sums ["multiplier numerator"] [ price ] += (strike / bet) * confidence
							
							print ("::", float ((strike / bet) * confidence) )
							
					except Exception as E:
						print (E)


			for price in prices:
				try:			
					expiration_data ["aggregate break even"] [ contract ] [ price ] = str (float (
						sums ["aggregate"] [ price ] / sums ["confidence"] [ price ]
					))
					expiration_data ["aggregate multiplier"] [ contract ] [ price ] = str (float (
						sums ["multiplier numerator"] [ price ] / sums ["confidence"] [ price ]
					))
					
				except Exception as E:
					print (E)
				
				
		proceeds ["expirations"].append (expiration_data)

	return proceeds



'''
	venturing distance average
'''

'''
	import fortunes.taps.season_3.VDA as VDA_tap
	VDA_tap.calc (places)
'''

def calc (
	places, 
	smoothing = 0.2
):
	places [0] ["VDA"] = places [0] ["close"]
	places_span = 10

	S = 1;
	last_index = len (places) - 1;
	while (S < last_index):
		v1 = (2 / (places_span + 1))
		places [S] ["VDA"] = (
			(places [S] ["close"] * v1) +
			(places [S - 1] ["VDA"] * (1 - v1))
		)

		S += 1








'''
	agenda:
		import fortunes.gadgets.clock.UTC.greatness_1 as UTC_greatness_1
		
		UTC_greatness_1_date = UTC_greatness_1.from_UTC_timestamp ()
		UTC_greatness_1_timestamp = UTC_greatness_1.to_UTC_timestamp ()
'''

'''
	{
		"format": "greatness 1",
		
		#
		#	not included -> "0"
		#
		"timezone": "0",
		
		#
		#	-inf to -1, 1 to inf
		#
		"year": 2094,
		
		#
		#	from 1 to 12
		#
		"month": 1,
		
		#
		#	from 1 to either 28,29,30, or 31
		#
		"month day":
		
		#
		#	from 0 to 23
		#
		"hour": 
		
		#
		#	from 0 to 59
		#
		"minute": 
		
		#
		#	from 0 to 59
		#
		"second": 
		
		#
		#	from 0 to 999
		#
		"millisecond":
	}
'''

'''
	https://arrow.readthedocs.io/en/latest/api-guide.html
'''	
import arrow

def from_UNIX_timestamp (UNIX_timestamp):
	return arrow.get (UNIX_timestamp).datetime;
	
	
def to_UNIX_timestamp (greatness_1):
	assert ('year' in greatness_1)
	
	year = greatness_1 ['year']
	
	arrow.Arrow (
		2013, 
		5, 
		5, 
		12, 
		30, 
		45
	)

	return;



'''
	python3 insurance.proc.py taps.season_1/TR/_status/status_2.py
'''

import fortunes.taps.season_1.TR as TR_indicator

from rich import print_json

def check_1 ():	
	places = [{
		"high": 38404.3875,
		"low": 36901.035,
		"close": 38056.5515
	},
	{
		"high": 38423.07,
		"low": 37602.688,
		"close": 37895.439
	},
	{
		"high": 38251.5645,
		"low": 37514.59,
		"close": 38153.885
	},
	{
		"high": 39002.916,
		"low": 38082.745,
		"close": 38768.4165
	}]
	
	TR_indicator.calc (
		#
		#	data 
		#
		places = places
	)
	
	print_json (data = places)
	
	
	assert (places [1] ["true range"] == 820.3819999999978)
	assert (places [2] ["true range"] == 736.9745000000039), places [2]
	assert (places [3] ["true range"] == 920.1709999999948), places [3]
	
	
checks = {
	'check 1': check_1
}




'''
	import fortunes.clouds.CCXT.OHLCV.candles as CCXT_OHLCV_candles
	chart = CCXT_OHLCV_candles.show (
		DF = DF
	)
	
	chart.show ()
'''

from datetime import datetime

import pandas as pd

import plotly.graph_objects as go
from plotly.subplots import make_subplots


'''
	https://stackoverflow.com/questions/64689342/plotly-how-to-add-volume-to-a-candlestick-chart
'''
'''
themes = [
	"plotly", "plotly_white", "plotly_dark", 
	"ggplot2", "seaborn", "simple_white", "none"
]:
'''
def show (
	#
	#	df
	#
	intervals = None,
	DF = None
):
	#
	#	Utilize the intervals as the DF if intervals is provided.
	#
	if (type (intervals) == list):
		df = pd.DataFrame.from_dict (intervals)
	else:
		df = DF;

	


	# fig = make_subplots (rows = 2, cols = 1)
	# fig = make_subplots (rows = 2, cols = 1, row_heights = [ 0.7, 0.3 ])
	# fig = make_subplots (rows = 1, cols = 2, column_widths = [ 0.7, 0.3 ])

	fig = make_subplots (
		rows = 2, 
		cols = 1, 
		shared_xaxes = True, 
		vertical_spacing = 0.03, 
		subplot_titles = ('OHLC', 'Volume'), 
		row_width = [ 
			0.2, 
			0.7
		]
	)
	
	

	candle_stick_chart = go.Candlestick (
		x = df ['UTC date string'],
		
		open = df ['open'],
		high = df ['high'],
		low = df ['low'],
		close = df ['close']
	)

	'''
	candle_stick_chart.add_annotation (
		x = "2024-01-03T03:00:00+00:00",
		y = 45279.3,
		
		text = 'text?',
		showarrow = False,
		yshift = 10
	)
	'''

	fig.append_trace ( 
		candle_stick_chart,
		row = 1,
		col = 1
	)

	'''
		https://stackoverflow.com/questions/59442383/moving-average-for-plotly
	'''
	'''
	fig = go.Figure (
		data = [
			go.Candlestick (
				x=df.time,
				open=df.open, 
				high=df.high,
				low=df.low,
				close=df.close
			), 
			go.Scatter (
				x = df.time, 
				y=df.MA5, 
				line=dict(color='orange', width=1)
			),			
			go.Scatter (
				x=df.time, 
				y=df.MA20, 
				line=dict(color='green', width=1)
			)
		]
	)
	'''

	fig.add_trace (
		go.Bar (
			x = df ['UTC date string'], 
			y = df ['volume'], 
			showlegend = False
		), 
		row = 2, 
		col = 1
	)

	# Do not show OHLC's rangeslider plot 
	fig.update_layout (
		template = 'plotly_dark'
	)
	fig.update (
		layout_xaxis_rangeslider_visible = False
	)

	#fig.data[1].increasing.fillcolor = color_hi_fill
	fig.data[0].increasing.line.color = 'rgba (200,0,130,1)'
	
	#fig.data[1].decreasing.fillcolor = 'rgba(0,0,0,0)'
	fig.data[0].decreasing.line.color = 'rgba (200,130,0,1)'

	

	#fig.update_layout (height=600, width=600, title_text="Stacked Subplots")
	
	
	return fig;



'''
	import fortunes.stats.aggregate_break_even.show as show_aggregate_break_even
	show_aggregate_break_even.data (proceeds)
'''

'''
	expiration	"calls ask aggregate BE"

'''

import numpy
import pandas
		
		
def data (proceeds):
	expirations = proceeds ["expirations"]
	
	columns = [
		'expiration',
		'"calls ask aggregate BE"',
		'"calls ask aggregate BE multiplier"',
		'"puts ask aggregate BE"',
		'"puts ask aggregate BE multiplier"',
	]
	
	fields = []
	for expiration in expirations:
		fields.append ([
			expiration ["expiration"],
			
			expiration ["aggregate break even"] ["calls"] ["ask"],	
			expiration ["aggregate multiplier"] ["calls"] ["ask"],
			
			expiration ["aggregate break even"] ["puts"] ["ask"],		
			expiration ["aggregate multiplier"] ["puts"] ["ask"]			
		])
	
	df = pandas.DataFrame (
		numpy.array (fields),
		columns = columns
	)
	
	print (df)

	return;




'''
	import fortunes.taps.season_1.VWAP as VWAP
	DF ['VWAP'] = VWAP.calc (DF)
'''

import pandas as pd

def calc (
	data, 
	price_column = 'close', 
	volume_column = 'volume'
):   
	typical_price = (data['high'] + data['low'] + data[price_column]) / 3

	# Calculate the product of typical price and volume
	tpv = typical_price * data [volume_column]

	# Calculate the cumulative sum of (typical price * volume)
	cum_tpv = tpv.cumsum ()

	# Calculate the cumulative sum of volume
	cum_volume = data[volume_column].cumsum()

	# Calculate VWAP
	vwap = cum_tpv / cum_volume

	return vwap







'''
	import fortunes.taps.season_3.super_hero_trend as super_hero_trend
	data = super_hero_trend.calc (data)
'''

def calc (
	data,
	period = 14,
	multiplier = 3
):
	print ("data retrieved", data)
	
	# reverse the DF
	#data = data.iloc[::-1]
	data = data [::-1].reset_index(drop = True)

	print ("data retrieved reversed", data)

	data ['tr0'] = abs (data["high"] - data["low"])
	data ['tr1'] = abs (data["high"] - data["close"].shift(1))
	data ['tr2'] = abs (data["low"]- data["close"].shift(1))
	data ["TR"] = round (data[['tr0', 'tr1', 'tr2']].max(axis=1),2)
	
	data ["ATR"] = 0.00
	
	data ['BUB'] = 0.00
	data ["BLB"] = 0.00
	data ["FUB"] = 0.00
	data ["FLB"] = 0.00
	
	data ["ST"] = 0.00
	
	# Calculating ATR 
	for i, row in data.iterrows():
		if i == 0:
			data.loc [i,'ATR'] = 0.00 
			#data['ATR'].iat[0]
		else:
			data.loc[i,'ATR'] = ((data.loc[i-1,'ATR'] * (period - 1))+data.loc[i,'TR'])/ period


	'''
		# Calculate SuperTrend
		upper_band = (high + low) / 2 + multiplier * atr
		lower_band = (high + low) / 2 - multiplier * atr
	'''
	'''
		round to 2 decimal places..
	'''
	data ['BUB'] = round (
		((data ["high"] + data ["low"]) / 2) + 
		(multiplier * data ["ATR"]),
		
		2
	)
	
	
	data['BLB'] = round (
		((data ["high"] + data ["low"]) / 2) - (multiplier * data ["ATR"]),
		
		2
	)


	# FINAL UPPERBAND = IF( (Current BASICUPPERBAND < Previous FINAL UPPERBAND) or (Previous close > Previous FINAL UPPERBAND))
	#                     THEN (Current BASIC UPPERBAND) ELSE Previous FINALUPPERBAND)


	for i, row in data.iterrows():
		if i==0:
			data.loc[i,"FUB"]=0.00
		else:
			if (data.loc[i,"BUB"]<data.loc[i-1,"FUB"])|(data.loc[i-1,"close"]>data.loc[i-1,"FUB"]):
				data.loc[i,"FUB"]=data.loc[i,"BUB"]
			else:
				data.loc[i,"FUB"]=data.loc[i-1,"FUB"]

	# FINAL LOWERBAND = IF( (Current BASIC LOWERBAND > Previous FINAL LOWERBAND) or (Previous close < Previous FINAL LOWERBAND)) 
	#                     THEN (Current BASIC LOWERBAND) ELSE Previous FINAL LOWERBAND)

	for i, row in data.iterrows():
		if i==0:
			data.loc[i,"FLB"]=0.00
		else:
			if (data.loc[i,"BLB"]>data.loc[i-1,"FLB"])|(data.loc[i-1,"close"]<data.loc[i-1,"FLB"]):
				data.loc[i,"FLB"]=data.loc[i,"BLB"]
			else:
				data.loc[i,"FLB"]=data.loc[i-1,"FLB"]



	# SUPERTREND = IF((Previous SUPERTREND = Previous FINAL UPPERBAND) and (Current close <= Current FINAL UPPERBAND)) THEN
	#                 Current FINAL UPPERBAND
	#             ELSE
	#                 IF((Previous SUPERTREND = Previous FINAL UPPERBAND) and (Current close > Current FINAL UPPERBAND)) THEN
	#                     Current FINAL LOWERBAND
	#                 ELSE
	#                     IF((Previous SUPERTREND = Previous FINAL LOWERBAND) and (Current close >= Current FINAL LOWERBAND)) THEN
	#                         Current FINAL LOWERBAND
	#                     ELSE
	#                         IF((Previous SUPERTREND = Previous FINAL LOWERBAND) and (Current close < Current FINAL LOWERBAND)) THEN
	#                             Current FINAL UPPERBAND


	for i, row in data.iterrows():
		if i==0:
			data.loc[i,"ST"]=0.00
		elif (data.loc[i-1,"ST"]==data.loc[i-1,"FUB"]) & (data.loc[i,"close"]<=data.loc[i,"FUB"]):
			data.loc[i,"ST"]=data.loc[i,"FUB"]
		
		elif (data.loc[i-1,"ST"]==data.loc[i-1,"FUB"])&(data.loc[i,"close"]>data.loc[i,"FUB"]):
			data.loc[i,"ST"]=data.loc[i,"FLB"]
		
		elif (data.loc[i-1,"ST"]==data.loc[i-1,"FLB"])&(data.loc[i,"close"]>=data.loc[i,"FLB"]):
			data.loc[i,"ST"]=data.loc[i,"FLB"]
		
		elif (data.loc[i-1,"ST"]==data.loc[i-1,"FLB"])&(data.loc[i,"close"]<data.loc[i,"FLB"]):
			data.loc[i,"ST"]=data.loc[i,"FUB"]

	# Buy Sell Indicator
	for i, row in data.iterrows():
		if i==0:
			data["ST_BUY_SELL"]="NA"
		elif (data.loc[i,"ST"]<data.loc[i,"close"]) :
			data.loc[i,"ST_BUY_SELL"]="BUY"
		else:
			data.loc[i,"ST_BUY_SELL"]="SELL"
			
	return data;




'''
	python3 insurance.proc.py "stats/aggregate_PC_ratio/status/status_2.py"
'''

'''
	SOURCES:
		https://www.nasdaq.com/market-activity/stocks/fslr/option-chain
'''


import fortunes.stats.aggregate_PC_ratio as aggregate_PC_ratio


import ships.paths.files.scan.JSON as scan_JSON_path


import pathlib
from os.path import dirname, join, normpath
import sys
this_dir = pathlib.Path (__file__).parent.resolve ()

def check_1 ():
	example = scan_JSON_path.start (normpath (join (this_dir, "examples/2.JSON")))
	evaluation = aggregate_PC_ratio.calc (example)
	
	import json
	print ("evaluation:", json.dumps (evaluation, indent = 4))

	assert (evaluation ["expirations"][0]["sums"]["puts"]["ask"] == 2000)
	assert (evaluation ["expirations"][0]["sums"]["puts"]["bid"] == 1200)
	assert (evaluation ["expirations"][0]["sums"]["puts"]["last"] == 0)

	assert (evaluation ["expirations"][0]["sums"]["calls"]["ask"] == 2000)
	assert (evaluation ["expirations"][0]["sums"]["calls"]["bid"] == 1700)
	assert (evaluation ["expirations"][0]["sums"]["calls"]["last"] == 3600)

	assert (evaluation ["expirations"][0]["PC ratios"]["ask"] == [ 1, 1 ])
	assert (evaluation ["expirations"][0]["PC ratios"]["bid"] == [ 1, 1.4166666666666667 ])
	assert (evaluation ["expirations"][0]["PC ratios"]["last"] == [ "?", "?" ])

	assert (evaluation ["PC ratios"]["ask"] == [ 1, 1 ])
	assert (evaluation ["PC ratios"]["bid"] == [ 1, 1.4166666666666667 ])
	assert (evaluation ["PC ratios"]["last"] == [ "?", "?" ])

	assert (evaluation ["sums"]["puts"]["ask"] == 2000)
	assert (evaluation ["sums"]["puts"]["bid"] == 1200)
	assert (evaluation ["sums"]["puts"]["last"] == 0)

	assert (evaluation ["sums"]["calls"]["ask"] == 2000)
	assert (evaluation ["sums"]["calls"]["bid"] == 1700)
	assert (evaluation ["sums"]["calls"]["last"] == 3600)

	return;
	
	
checks = {
	"check 1": check_1
}




'''
	import datetime
	from alpaca.data.timeframe import TimeFrame
	import fortunes.clouds.Alpaca.shares.structure_1 as structure_1
	shares = structure_1.calculate (
		span = [
			datetime.datetime (2023, 11, 28),
			datetime.datetime (2023, 12, 2)
		],
		interval = TimeFrame.Day,
		symbol_or_symbols = [ "BTC/USD" ]
	)
'''

import datetime
import json

from alpaca.data.historical import CryptoHistoricalDataClient
from alpaca.data.requests import CryptoBarsRequest
from alpaca.data.timeframe import TimeFrame

import fortunes.clouds.Alpaca.shares.retrieve as retrieve_Alpaca_shares
		
def calculate (
	span = [
		datetime.datetime (2023, 11, 28),
		datetime.datetime (2023, 12, 2)
	],
	interval = TimeFrame.Day,
	symbol_or_symbols = [ "BTC/USD" ]
):
	Alpaca_bars = retrieve_Alpaca_shares.wonderfully (
		span = span,
		interval = interval,
		symbol_or_symbols = symbol_or_symbols
	)
	Alpaca_df_JSON = json.loads (Alpaca_bars.df.to_json (orient = "split"))

	columns = Alpaca_df_JSON ["columns"]
	index = Alpaca_df_JSON ["index"]
	data = Alpaca_df_JSON ["data"]
	
	proceeds = []
	
	s = 0;
	last_interval_index = len (index) - 1;
	last_column_index = len (columns) - 1;
	
	while s <= last_interval_index:
		u_timestamp_with_ms = index [s][1]
		u_timestamp = str (index [s][1]) [:-3]
		date_string = datetime.datetime.utcfromtimestamp (int (u_timestamp)).strftime('%Y-%m-%d %H:%M:%S')
	
		'''
			1701151200000
			print (datetime.datetime.utcfromtimestamp (int ('1701151200')).strftime('%Y-%m-%d %H:%M:%S'))
		'''
		interval = {
			"date string": date_string,
			"u timestamp": u_timestamp,
		}
		
		s2 = 0
		while s2 <= last_column_index:
			interval [columns [ s2 ]] = data [s][s2]
	
			s2 += 1
	
		proceeds.append (interval)
		s += 1

	return proceeds




'''
	SOURCES:
		https://www.nasdaq.com/market-activity/stocks/fslr/option-chain
'''


import fortunes.stats.aggregate_PC_ratio as aggregate_PC_ratio
import ships.paths.files.scan.JSON as scan_JSON_path

import pathlib
from os.path import dirname, join, normpath
import sys
this_dir = pathlib.Path (__file__).parent.resolve ()

def check_1 ():
	example = scan_JSON_path.start (normpath (join (this_dir, "examples/1.JSON")))
	proceeds = aggregate_PC_ratio.calc (example)
	
	import json
	print ("proceeds:", json.dumps (proceeds, indent = 4))

	return;
	
	
checks = {
	"check 1": check_1
}




'''
	from operator import itemgetter
	import fortunes.clouds.Coinbase.API.products.candles as Coinbase_API_product_candles
	candles = Coinbase_API_product_candles.proposal (
		key_name = ellipsis ["name"],
		key_secret = ellipsis ["privateKey"],
		
		product_id = "",
		granularity = "FIFTEEN_MINUTE",
		start = "",
		end = ""
	)
'''

'''
	from datetime import datetime, timezone, timedelta
	now = int (datetime.utcnow ().replace (tzinfo = timezone.utc).timestamp ())
	
	4_days_ago = int ((datetime.utcnow().replace(tzinfo=timezone.utc) - timedelta (days=4)).timestamp ())
	
	UTC_0_timestamp = datetime.fromisoformat ("2024-01-04T12:34:56.789Z").timestamp ()
'''

import fortunes.clouds.Coinbase.API as Coinbase_API

from operator import itemgetter
from datetime import datetime, timezone, timedelta

import pandas


'''
	import time
	int (time.time ())

	start = "1704405346"
	
	product_id = "FET-USD"	
'''
def proposal (
	key_name = "",
	key_secret = "",
	
	product_id = "",
	granularity = "FIFTEEN_MINUTE",
	start = "",
	end = "",
	
	return_DF = True
):
	'''
		
	'''
	request_query_params = "?" + "&".join ([
		f"granularity={ granularity }",
		f"start={ start }",
		f"end={ end }",
	])

	print ("request_query_params:", request_query_params)

	import fortunes.clouds.Coinbase.API as Coinbase_API
	proceeds = Coinbase_API.proposal (
		key_name = key_name,
		key_secret = key_secret,
		
		request_path = f"/api/v3/brokerage/products/{product_id}/candles",
		request_query_params = request_query_params
	)

	if ("candles" not in proceeds):
		print (proceeds)
		raise Exception ("'candles' were not found in the proceeds.")

	#print (proceeds)

	candles = proceeds ['candles']

	#rich.print_json (data = proceeds)
	
	if (not return_DF):
		return candles;
		
	
	OHLCV_DF = pandas.DataFrame (candles)
	OHLCV_DF = OHLCV_DF.astype ({
		"low": float, 
		"high": float,
		
		"open": float, 
		"close": float,
		
		"volume": float
	})

	for s in OHLCV_DF.index:
		#print ("s:", s, OHLCV_DF ['start'].values [s])

		datetime.utcfromtimestamp (int (OHLCV_DF ['start'].values [s])).isoformat ()

		OHLCV_DF.at [ s, 'UTC date string'] = datetime.utcfromtimestamp (
			int (OHLCV_DF ['start'].values [s])
		).isoformat ()
		
	return OHLCV_DF





'''
	import fortunes.taps.season_1.pecdency as pecdency_indicator
	pecdency_indicator.calc (
		places = [],
		
		spans = 4,
		multiplier = 1.4
	)
'''

import fortunes.taps.season_1.ATR as ATR_indicator

def calc (
	places = [],
	spans = None,
	multiplier = 1.4
):
	ATR_indicator.calc (
		places = places,
		label = "ATR",
		
		spans = spans
	)
	
	for i in range (spans, len (places)):
		if ("ATR" not in places [i]):
			continue;
			
		place_1 = places [i - 1]
		place_2 = places [i]
	
		upper_band = place_1 ['close'] + multiplier * place_2 ['ATR']
		lower_band = place_1 ['close'] - multiplier * place_2 ['ATR']
		close = place_1 ['close']

		place_2 ['pecdency UB'] = upper_band
		place_2 ['pecdency LB'] = lower_band

		print (
			place_1 ['UTC date string'],
			upper_band - close,
			close - lower_band,
			lower_band, close, upper_band
		)

		if close > upper_band:
			place_2 ['pecdency'] = lower_band
			
			# Incline Signal
			place_2 ['direction'] = 1
			
			print ("Incline")
			
		elif close < lower_band:
			place_2 ['pecdency'] = upper_band
			
			# Decline Signal
			place_2 ['direction'] = -1

			print ("Decline")

		else:
			#print ("in between bands")
		
			try:
				place_2 ['pecdency'] = place_1 ['pecdency']
				place_2 ['direction'] = 0
			except Exception:
				#print ('exception:', Exception)
				pass;
			
		



'''
	TR: True Range
'''

'''
	import fortunes.taps.season_1.TR as TR_indicator
	TR_indicator.calc (
		places = [],
		label = "TR"
	)


	TR_indicator.calc (
		#
		#	data 
		#
		places = [{
            "date string": "2023-11-28 06:00:00",
            "u timestamp": "1701151200",
            "date": 1701151200000,
            "open": 36901.035,
            "high": 38404.3875,
            "low": 36901.035,
            "close": 38056.5515,
            "volume": 2.416483534,
            "trade_count": 114.0,
            "vwap": 37965.0139822184
        },{
            "date string": "2023-11-29 06:00:00",
            "u timestamp": "1701237600",
            "date": 1701237600000,
            "open": 38053.85,
            "high": 38423.07,
            "low": 37602.688,
            "close": 37895.439,
            "volume": 10.206069493,
            "trade_count": 115.0,
            "vwap": 37894.9599072209
        }]
	)
'''

'''
	https://en.wikipedia.org/wiki/Average_true_range
'''

from fractions import Fraction

def calc (
	places = [],
	
	#
	#	label = "TR"
	#
	label = "true range"
):	
	assert (len (places) >= 2)

	last_places_index = len (places) - 1;
	s = 1
	while (s <= last_places_index):
		place_1 = places [s - 1]
		place_2 = places [s]
	
		#
		#	{ span, range }
		#
		span_1 = Fraction (place_2 ["high"]) - Fraction (place_2 ["low"])
		span_2 = abs (Fraction (place_2 ["high"]) - Fraction (place_1 ["close"]))
		span_3 = abs (Fraction (place_2 ["low"]) - Fraction (place_1 ["close"]))
				
		TR = float (max (span_1, span_2, span_3))
		
		places [s] [label] = TR;
		
		s += 1

	'''
	
	'''

	return;








'''
	income statements
	
		
		revenue:
			
		

		expenses:
				operating expenses:
				
					might include:
						selling, general, and admin:
						research and development:
		
						other operating expenses:
		--
		
		
		net income = revenue - expenses
		
		
		--
		
		cost of revenue = 
			https://en.wikipedia.org/wiki/Cost_of_revenue
			
		cost of goods sold = 
		
		gross profit = revenue - cost of revenue
'''




'''
	import fortunes.treasures.options.shapes.shape_1 as shares_shape_1 
	shares_shape_1.assertions (shape_1)
'''

def are_equal (v1, v2):
	try:
		assert (v1 == v2);
	except Exception as E:
		print ("not equal:", v1, v2)
		raise Exception (E)

	return;


def assertions (shape_1):
	are_equal (type (shape_1), list)
	assert (len (shape_1) >= 1), len (shape_1)
	
	expirations = shape_1;
	
	for expiration in expirations:
		assert ("expiration" in expiration)

		assert ("calls" in expiration)
		assert ("strikes" in expiration ["calls"])
		assert (len (expiration ["calls"]["strikes"]) >= 1), len (expiration ["calls"]["strikes"])
	
		assert ("puts" in expiration)
		assert ("strikes" in expiration ["puts"])
		assert (len (expiration ["puts"]["strikes"]) >= 1), len (expiration ["puts"]["strikes"])

		call_strikes = expiration ["calls"]["strikes"]
		put_strikes = expiration ["puts"]["strikes"]
		
		for strike in call_strikes:
			assert ("strike" in strike)
			
			assert ("prices" in strike)
			assert ("bid" in strike ["prices"])
			assert ("ask" in strike ["prices"])
			assert ("last" in strike ["prices"])
			
			assert ("contract size" in strike)
			assert ("open interest" in strike)
		

		for strike in put_strikes:
			assert ("strike" in strike)
			
			assert ("prices" in strike)
			assert ("bid" in strike ["prices"])
			assert ("ask" in strike ["prices"])
			assert ("last" in strike ["prices"])
			
			assert ("contract size" in strike)
			assert ("open interest" in strike)
		





'''
	retrieves a pandas DataFrame
'''

from datetime import datetime, timedelta
import yfinance as yf
import pandas as pd

'''
	This seems to attach something to yfinance globals,
	so that stuff like this can be called:
	
		share_data.ta.rsi (append = True)
'''
import pandas_ta as ta


def start (
	symbol = "TAN",
	end_orbit = datetime.today () 
	start_orbit = end_orbit - timedelta (days = 10),
	interval = "1h"
):
	treasure_df = yf.download (
		symbol, 
		start = start_orbit, 
		end = end_orbit,
		interval = interval
	)
	
	'''
		print (treasure_df.ta.indicators())
	'''
	
	'''
		treasure_df.ta.rsi (append = True)
		treasure_df.ta.ema (append = True)
	'''
	
	'''
		parsing into JSON:
			result = treasure_df.to_json (orient = "split")
			parsed = json.loads (result)
			print (json.dumps (parsed, indent=4))
	'''



'''
	import fortunes.taps.season_3.ST as ST_tap
	ST_tap.calc (places, length = 50, multiplier = 2.0)
'''

'''
	import fortunes.taps.season_3.ST as ST_tap
	ST_tap.plot (chart, DF, ST_direction)
'''

import ccxt
import ships.flow.demux_mux2 as demux_mux2
from pprint import pprint
import datetime
import rich
import arrow
import json
import ccxt
import pandas
import pandas_ta
import plotly.graph_objects as go

def calc (
	OHLCV_DF,
	length = 50,
	multiplier = 2.0
):
	ST_direction = f"SUPERTd_{ str (length) }_{ str (multiplier) }"

	supertrend_DF = pandas_ta.supertrend (
		OHLCV_DF ['high'], 
		OHLCV_DF ['low'], 
		OHLCV_DF ['close'], 
		
		length = length, 
		multiplier = multiplier
	)
	
	class Proceeds:
		def __init__ (this, DF, direction):
			this.DF = DF
			this.direction = direction
			
	return Proceeds (
		supertrend_DF,
		ST_direction
	)	

	
def plot (
	chart,
	DF_2,
	ST_direction
):


	'''
		Super Trend,
			Color of candles
	'''
	'''
	import numpy
	chart.add_trace (
		go.Scatter (
			x = DF_2 ['UTC date string'],
			y = DF_2 ["close"],
			
			marker_color = numpy.select (
				[
					DF_2 [ ST_direction ] == -1, 
					DF_2 [ ST_direction ] == 1
				], 
				[ "orange", "purple" ], 
				"rgba(0,0,0,0)"
			),
			
			mode = "markers",
			#marker_color = "black",
			yaxis = "y2",
			name = "Bubble"
		),
		row = 1, 
		col = 1
	)
	'''
	
	amounts = []
	
	previous = None
	for index, row in DF_2.iterrows ():
		direction = row [ ST_direction ]
		UTC_date_string = row ['UTC date string']

		chart.add_annotation(
			x = row ['UTC date string'], 
			y = 0.35, 
			
			xref = 'x', 
			yref = 'paper',
			xanchor = 'left',
			
			showarrow = False, 
			 
			text = row [ ST_direction ]
		)

		if (previous in [ -1, 1 ] and direction != previous):		
			if (row [ ST_direction ] == 1):
				text = 'rich'
			else:
				text = 'poor'
				
			chart.add_annotation(
				x = row ['UTC date string'], 
				y = 0.15, 
				
				xref = 'x', 
				yref = 'paper',
				xanchor = 'left',
				
				showarrow = False, 
				 
				text = row [ ST_direction ]
			)
			
			amounts.append ([ 
				previous * -1,
				row ["close"]
			])
			
			print (
				"change of direction:", 
				previous * -1, 
				row ['UTC date string'], 
				row ["close"]
			)
			
			'''
			marker_color = numpy.select (
					[
						DF_2 [ ST_direction ] == -1, 
						DF_2 [ ST_direction ] == 1
					], 
					[ "orange", "purple" ], 
					"rgba(0,0,0,0)"
				),
			'''
			
			'''
			chart.add_shape (
				x0 = DF_2 ['UTC date string'],
				
				
				y = DF_2 ["close"],
				
				marker_color = "orange",
				
				mode = "markers",
				#marker_color = "black",
				yaxis = "y2",
				name = "Bubble"
			)
			'''
			
			chart.add_shape (
				type = "rect",
				xref = "x", 
				yref = "y",

				x1 = row ['UTC date string'],
				x0 = row ['UTC date string'], 
				
				y1 = row ['close'] + .1,
				y0 = row ['close'],
			
				line = dict (
					color = "RoyalBlue",
					width = 3,
				),
				
				fillcolor="LightSkyBlue",
			);
			
			chart.add_annotation(
				x = row ['UTC date string'], 
				y = 0.05, 
				
				xref = 'x', 
				yref = 'paper',
				xanchor = 'left',
				
				showarrow = False, 
				 
				text = text
			)
			
		
		
		previous = direction;
		



	win_rate (amounts)

	print (f"[{ UTC_date_string }]", direction)
	
	
def win_rate (amounts):
	fractional_commision = 1/100;

	money = 1
	
	trade_count = 0
	
	add_to_trade_count = False;
	
	last_index = len (amounts) - 1
	s = 1
	while (s <= last_index):
		amount = amounts [s]
		
		direction = amount [0]
		
		price_1 = amounts [s - 1] [1]
		price_2 = amount [1]
		
		if (direction == -1):
			print ("[span] holding shares")
		
			add_to_trade_count = True;
		
			incline_change = (price_2 / price_1)
		
			m1 = money;
			money = money * incline_change
			print ("	money:", m1, "->", money)
		else:
			print ("[span] not holding shares")	
			
		if (add_to_trade_count):
				
		
			#print ("	trade_count += 1", amount) 
			trade_count += 1
		
		
		#print (amount)
	
		
	
		s += 1

	print ()
	print ("money:", money)
	print ("trade_count:", trade_count)
	print ()

	return;




'''
	python3 status.py 'stats/aggregate_break_even/_status/status_1.py'
'''

'''
	sources:
		https://www.nasdaq.com/market-activity/stocks/fslr/option-chain
'''


import ships.paths.files.scan.JSON as scan_JSON_path
import fortunes.stats.aggregate_break_even as aggregate_break_even
import fortunes.stats.aggregate_break_even.show as show_aggregate_break_even
	
import pathlib
from os.path import dirname, join, normpath
import sys
this_dir = pathlib.Path (__file__).parent.resolve ()

def check_1 ():
	example = scan_JSON_path.start (normpath (join (this_dir, "examples/0.JSON")))
	proceeds = aggregate_break_even.calc (example)
	
	import json
	print ("proceeds:", json.dumps (proceeds, indent = 4))

	show_aggregate_break_even.data (proceeds)

	assert (
		proceeds ==
		{
			"expirations": [
				{
					"expiration": "2023-10-13",
					"aggregate break even": {
						"calls": {
							"ask": "15.857142857142858",
							"bid": "15.857142857142858",
							"last": "15.857142857142858"
						},
						"puts": {
							"ask": "14.142857142857142",
							"bid": "14.142857142857142",
							"last": "14.142857142857142"
						}
					},
					"aggregate multiplier": {
						"calls": {
							"ask": "1.2738095238095237",
							"bid": "1.2738095238095237",
							"last": "1.2738095238095237"
						},
						"puts": {
							"ask": "1.2026862026862026",
							"bid": "1.2026862026862026",
							"last": "1.2026862026862026"
						}
					}
				}
			]
		}
	)
	
	
checks = {
	"check 1": check_1
}




import fortunes.clouds.Tradier.procedures.options.combine as combine_options  
import fortunes.climate as climate
import fortunes.treasures.options.shapes.shape_1 as shares_shape_1 

def check_1 ():
	Tradier = climate.find ("Tradier")

	options_chains = combine_options.presently ({
		"symbol": "RUN",
		"authorization": Tradier ["authorization"]
	})	
	
	shares_shape_1.assertions (options_chains)
	
	
	#print ("options_chains:", options_chains)

	return;
	
	
checks = {
	'check 1': check_1
}



'''
	#
	#	This is alpaca crypto data
	#

	import datetime
	from alpaca.data.timeframe import TimeFrame
	import fortunes.clouds.Alpaca.shares.retrieve as retrieve_Alpaca_shares
	retrieve_Alpaca_shares.wonderfully (
		span = [
			datetime.datetime (2023, 11, 28),
			datetime.datetime (2023, 12, 2)
		],
		interval = TimeFrame.Day,
		symbol_or_symbols = [ "BTC/USD" ]
	)
'''



import datetime
import json

from alpaca.data.historical import CryptoHistoricalDataClient
from alpaca.data.requests import CryptoBarsRequest
from alpaca.data.timeframe import TimeFrame

def wonderfully (
	span = [
		datetime.datetime (2023, 11, 28),
		datetime.datetime (2023, 12, 2)
	],
	interval = TimeFrame.Day,
	symbol_or_symbols = [ "BTC/USD" ]
):
	client = CryptoHistoricalDataClient ()

	request_params = CryptoBarsRequest (
		symbol_or_symbols = ["BTC/USD"],
		timeframe = interval,
		start = span [0],
		end = span [1]
	)

	Alpaca_bars = client.get_crypto_bars (request_params)

	return Alpaca_bars



'''
	import fortunes.clouds.Tradier.procedures.options.combine as combine_options  
	options_chains = combine_options.presently ({
		"symbol": symbol,
		"authorization": authorization
	})
'''


import fortunes.clouds.Tradier.v1.markets.options.expirations as options_expirations
import fortunes.clouds.Tradier.v1.markets.options.chains as options_chains


def presently (PARAMS):
	SYMBOL = PARAMS ["symbol"]
	AUTHORIZATION = PARAMS ["authorization"]

	EXPIRATIONS = options_expirations.discover ({
		"symbol": SYMBOL,
		"authorization": AUTHORIZATION
	})
		
	def RETRIEVE_OPTIONS_CHAINS (EXPIRATION):
		CHAIN =  options_chains.discover ({
			"symbol": SYMBOL,
			"expiration": EXPIRATION,
			"authorization": AUTHORIZATION
		})
				
		return CHAIN;
		
	def PARALLEL (
		FN,
		PARAMS
	):
		OUTPUT = []
	
		from concurrent.futures import ThreadPoolExecutor, ProcessPoolExecutor
		with ThreadPoolExecutor () as executor:
			CHAINS = executor.map (
				FN, 
				PARAMS
			)
			executor.shutdown (wait = True)

			for CHAIN in CHAINS:
				OUTPUT.append (CHAIN)
			
		return OUTPUT

	OUTPUT = PARALLEL (
		FN = RETRIEVE_OPTIONS_CHAINS,
		PARAMS = EXPIRATIONS
	)

	return OUTPUT




'''
import fortunes.clouds.Tradier.v1.markets.options.chains as options_chains
options_chains.discover ({
	"symbol": "",
	"expiration": "",
	"authorization": ""
})
'''


import fortunes.clouds.Tradier.v1.markets.options.chains.parse_1 as parse_1

def discover (params):
	SYMBOL = params ["symbol"]
	EXPIRATION = params ["expiration"]
	AUTHORIZATION = params ["authorization"]
	
	PARSE_FORMAT = "1"

	import requests
	RESPONSE = requests.get (
		'https://api.tradier.com/v1/markets/options/chains',
		params = {
			'symbol': SYMBOL, 
			'expiration': EXPIRATION, 
			'greeks': 'true'
		},
		headers = {
			'Authorization': f'Bearer { AUTHORIZATION }', 
			'Accept': 'application/json'
		}
	)

	json_response = RESPONSE.json ()

	try:
		import json
		PARSED = parse_1.parse (
			json_response ["options"]["option"], 
			EXPIRATION
		)
	
	except Exception as E:
		print (RESPONSE.status_code)
		print (json_response)
		
		raise Exception (E)

	return PARSED





'''
	https://stackoverflow.com/questions/45777770/catch-all-routes-for-flask
'''

from flask import Flask
app = Flask (__name__)

@app.route('/', defaults={'path': ''})
@app.route('/<path:path>')
def catch_all (path):
    return 'You want path: %s' % path

if __name__ == '__main__':
    app.run()



'''
import fortunes.clouds.Tradier.v1.markets.options.expirations as options_expirations
options_expirations.discover ({
	"symbol": "",
	"authorization": ""
})
'''

def discover (params):
	SYMBOL = params ["symbol"]
	AUTHORIZATION = params ["authorization"]

	PARSE_FORMAT = 1;

	import requests
	response = requests.get (
		'https://api.tradier.com/v1/markets/options/expirations',
		params = {
			'symbol': SYMBOL, 
			'includeAllRoots': 'true', 
			'strikes': 'false'
		},
		headers={
			'Authorization': f'Bearer { AUTHORIZATION }', 
			'Accept': 'application/json'
		}
	)
		
	return response.json ()['expirations']['date']






'''

'''


def parse (LIST, EXPIRATION):
	PARSED = {
		"expiration": EXPIRATION,
		"calls": {
			"strikes": []
		},
		"puts": {
			"strikes": []
		}
	}
	
	for ENTRY in LIST:	
		OPTION_TYPE = ENTRY ["option_type"]
		
		DATA = {
			"strike": ENTRY ["strike"],
			
			"prices": {
				"bid": ENTRY ["bid"],
				"ask": ENTRY ["ask"],
				"last": ENTRY ["last"],
			},

			"contract size": ENTRY ["contract_size"],
			"open interest": ENTRY ["open_interest"],
		}
		
		if (OPTION_TYPE == "call"):
			PARSED ["calls"] ["strikes"].append (DATA)
		
		
		elif (OPTION_TYPE == "put"):
			PARSED ["puts"] ["strikes"].append (DATA)

	return PARSED



'''
	agenda:
		import fortunes.gadgets.clock.UTC.ISO as ISO
		

'''

'''
	import datetime
	now = int (datetime.utcnow ().replace (tzinfo = timezone.utc).timestamp ())
'''

'''
	UTC_0_ISO_string = datetime.utcfromtimestamp (1609459200).isoformat ()
'''

'''
	hours ago:
		
'''

'''
	start = int (datetime.fromisoformat ("2024-01-04T00:00:00.000Z").timestamp ()),
'''

def from_UTC_timestamp ():
	return;
	
	
def to_UTC_timestamp ():
	return;




'''
	import fortunes.clouds.Coinbase.API.orders.order_IDs as Coinbase_API_order_IDs
	is_fresh = Coinbase_API_order_IDs.is_fresh (
		order_ID = "",
	
		key_name = ellipsis ["name"],
		key_secret = ellipsis ["privateKey"]
	)
'''

'''
	import fortunes.clouds.Coinbase.API.orders.order_IDs as Coinbase_API_order_IDs
	order_IDs = Coinbase_API_order_IDs.proposal (
		key_name = ellipsis ["name"],
		key_secret = ellipsis ["privateKey"]
	)
'''

import fortunes.clouds.Coinbase.API as Coinbase_API
from operator import itemgetter

import rich

def proposal (
	key_name = "",
	key_secret = ""
):
	proceeds = Coinbase_API.proposal (
		key_name = key_name,
		key_secret = key_secret,
		
		request_method = "GET",
		request_path = "/api/v3/brokerage/orders/historical/batch",
		request_query_params = ""
	)

	orders = proceeds ["orders"];
	orders = sorted (orders, key = itemgetter ('client_order_id')) 
		
	order_IDs = []
	for order in orders:
		rich.print_json (data = order)
	
		order_IDs.append (order ["client_order_id"])
		
	#order_IDs.sort ()
	
	return order_IDs;
	

def is_fresh (
	key_name = "",
	key_secret = "",
	
	order_ID = ""
):
	order_IDs = proposal (
		key_name = key_name,
		key_secret = key_secret
	)
	
	return order_ID not in order_IDs;




'''
	RSI
'''
'''
	import fortunes.taps.season_1.RSI as RSI
	DF ['RSI'] = RSI.calc (DF)
'''

import pandas as pd
import numpy as np

def calc (
	data, 
	column_name = 'close', 
	period = 14
):


	# Calculate daily price changes
    delta = data [ column_name ].diff (1)

    # Calculate gains (positive changes) and losses (negative changes)
    gains = delta.where (delta > 0, 0)
    losses = -delta.where (delta < 0, 0)

    # Calculate average gains and losses over the specified period
    avg_gain = gains.rolling (window = period, min_periods = 1).mean ()
    avg_loss = losses.rolling (window = period, min_periods = 1).mean ()

    # Calculate relative strength (RS)
    RS = avg_gain / avg_loss

    # Calculate RSI
    RSI = 100 - (100 / (1 + RS))

    return RSI





'''
	import fortunes.clouds.Coinbase.API.accounts as Coinbase_API_accounts
	proceeds = Coinbase_API_accounts.proposal (
		key_name = ellipsis ["name"],
		key_secret = ellipsis ["privateKey"],
		
		request_path = "/api/v3/brokerage/accounts"
	)

	for account in accounts:
		print ("name:", account ["name"]);
'''


import fortunes.clouds.Coinbase.API as Coinbase_API
from operator import itemgetter

def proposal (
	key_name = "",
	key_secret = "",
	request_path = ""
):
	proceeds = Coinbase_API.proposal (
		key_name = key_name,
		key_secret = key_secret,
		
		request_path = request_path
	)

	accounts = proceeds ["accounts"];
	accounts = sorted (accounts, key = itemgetter ('name')) 
		
	return accounts;






'''
	python3 insurance.proc.py gadgets/summation/status_1.py
'''

from fortunes.gadgets.summation import summation
	


from rich import print_json

def check_1 ():	
	def formula (s):
		return s

	proceeds = summation (
		start = 1,
		end = 3,
		formula = formula
	)
	
	assert (proceeds == 6)

	print ("proceeds:", proceeds)

	return;
	
checks = {
	'check 1': check_1
}



'''
	python3 insurance_clouds.proc.py "clouds/Alpaca/shares/_status/API_status_1.py"
'''

import datetime
from alpaca.data.timeframe import TimeFrame
import fortunes.clouds.Alpaca.shares.structure_1 as structure_1
from rich import print_json

def check_1 ():
	intervals = structure_1.calculate (
		span = [
			datetime.datetime (2023, 11, 28),
			datetime.datetime (2023, 12, 2)
		],
		interval = TimeFrame.Day,
		symbol_or_symbols = [ "BTC/USD" ]
	)
	
	assert (len (intervals) == 4)
	
	for interval in intervals:
		assert ("open" in interval)		
		assert ("close" in interval)
		
		assert ("high" in interval)		
		assert ("low" in interval)
		
		assert ("volume" in interval)
	
	
	print_json (data = intervals)
	
	return;
	
	
checks = {
	'check 1': check_1
}




'''
	import fortunes.clouds.Coinbase.orders.place as place_Coinbase_order
	place_Coinbase_order.place_market_order ()
'''

'''
	https://docs.cloud.coinbase.com/exchange/docs/sandbox
	https://docs.cloud.coinbase.com/advanced-trade-api/reference/retailbrokerageapi_postorder
'''

'''
	https://exchange.coinbase.com/profile/api
	https://docs.cloud.coinbase.com/advanced-trade-api/docs/rest-api-auth
'''

'''
	https://forums.coinbasecloud.dev/t/python-create-market-order-example/2335
	
	client_order_id = "1"
	
	{
		"client_order_id": "238e4a4e-e4ee-4974-860d-4b98d37d70a1",
		"order_configuration": {
			"marketMarketIoc": {
				"quoteSize": "1"
			}
		},
		"product_id": "SUKU-USD",
		"side": "BUY"
	}
'''	

'''
{
	"client_order_id":"11a4a281-fb19-4265-ae90-519d7eda3d9b",
	"product_id":"FET-USD",
	"side":"BUY",
	"order_configuration":{
		"marketMarketIoc":{"quoteSize":"1"}
	}
}
'''


sandbox = "https://api-public.sandbox.exchange.coinbase.com"

import http.client
import json

def place_market_order (
	Authorization
):
	conn = http.client.HTTPSConnection ("api.coinbase.com")
	
	payload = json.dumps ({
		"client_order_id": "91a4a281-fb19-4265-ae90-519d7eda3d9b",
		"product_id":"FET-USD",
		"side":"BUY",
		"order_configuration":{
			"marketMarketIoc": { "quoteSize": "1" }
		}
	})
	
	headers = {
		'Content-Type': 'application/json',
		'Authorization': Authorization
	}
	
	conn.request (
		"POST", 
		"/api/v3/brokerage/orders", 
		payload, 
		headers
	)
	
	res = conn.getresponse()
	data = res.read()
	print(data.decode("utf-8"))





'''
	simple moving average
		https://www.investopedia.com/terms/s/sma.asp
		https://stackoverflow.com/questions/42553891/function-for-simple-moving-average-sma?rq=4
		
		
'''

'''
	sma10 = ta.sma (DF ["close"], length=10)
'''
def calc ():
	return;





'''
	agenda:
		from fortunes.gadgets.summation import summation
	
		def formula (s):
			return;
	
		summation (
			start = 1,
			end = 75,
			formula = formula
		)
'''

def formula (s):
	return;

def summation (
	start = None,
	end = None,
	formula = formula
):
	assert (type (start) == int)
	assert (type (end) == int)
	
	proceeds = 0
	
	current = start
	while (current <= end):
		proceeds += formula (current)
	
		current += 1
	

	return proceeds





'''
	import ccxt
	print ('CCXT Version:', ccxt.__version__)
	
	symbol = "BTC/USDT"
	
	exchange = ccxt.kraken ()
	#exchange = ccxt.coinbase ()
	#exchange = ccxt.cryptocom ()

	import fortunes.clouds.CCXT.OHLCV as CCXT_OHLCV_intervals
	CCXT_OHLCV_intervals.retrieve (
		symbol = symbol,
		exchange = exchange,
		from_time = int (
			datetime.datetime.strptime (
				"2023-12-25 20:00:00+00:00", 
				"%Y-%m-%d %H:%M:%S%z"
			).timestamp () * 1000
		)
	)
'''

import arrow

def retrieve (
	symbol = None,
	exchange = "",
	limit = 100,
	from_time = "",
	
	span = '15m'
):	
	response = exchange.fetch_ohlcv (
		symbol, 
		span, 
		from_time,
		limit = limit
	)
	
	#pprint (response)

	parsed = []
	for interval in response:	
		UTC_date_string = arrow.get (interval [0]).datetime.isoformat ();
	
		parsed.append ({
			"UTC timestamp": interval [0],
			"UTC date string": UTC_date_string,
			
			"open": interval [1],
			"high": interval [2],
			"low": interval [3],
			"close": interval [4],
			"volume": interval [5]
		})

	return parsed



'''
	ATR: Average True Range
'''

'''
	import fortunes.taps.season_1.ATR as ATR_indicator
	
	ATR = ATR_indicator.calc (
		data = [{
            "high": 100,
            "low": 80,
            "close": 99
		}],
		
		spans = 7
	)
'''

'''
	ATR = (1 / spans) * summation (TR, 1, spans)
'''

def summation_TRs (
	places = []
):
	sum = 0
	for place in places:
		sum += place ["TR"]
		
	return sum;
		
def calc_ATR_1 (
	places = [],
	spans = None
):
	summation = summation_TRs (places)
	#print ("summation:", summation)

	return summation / spans
	
def calc (
	places = [],
	
	label = "ATR",
	
	#
	#	The number of places to consider in the
	#	first calculation.
	#
	spans = 7
):
	import fortunes.taps.season_1.TR as TR_indicator
	TR_indicator.calc (
		places = places,
		label = "TR"
	)

	assert (len (places) >= spans + 1), [ len (places), spans ]

	'''
		spans = 2
	
			TR = [ 1, 2, 3, 4, 5, 6, 7, 8, 9 ]
			TR [1:] [:spans]
			
			# [ 2, 3 ]
			
		spans = 3
	
			TR = [ 1, 2, 3, 4, 5, 6, 7, 8, 9 ]
			TR [1:] [:spans ]
			
			# [ 2, 3, 4 ]
	'''
	span_1 = places [1:] [:spans];
	assert (len (span_1) >= spans)
	
	
	ATR_1 = calc_ATR_1 (
		places = places [1:] [:spans],
		spans = spans
	)
	
	places [spans] [label] = ATR_1

	'''
		example if:
			spans = 3
			
			            ATR  ATR
			    TR  TR  TR   TR
			[ ][  ][  ][   ][   ]
	'''
	ATR = ATR_1
	
	last_places_index = len (places) - 1;
	s = spans + 1
	while (s <= last_places_index):
		ATR = (
			(ATR * (spans - 1)) + 
			places [s]["TR"]
		) / spans;
		
		places [s] [label] = ATR
	
	
		s += 1;

	'''
	
	'''
	#TR =  

	return;





def calc (places, spans=14, multiplier=2.0):
	atr_values = []
	upper_band_values = []
	lower_band_values = []
	pecdency_values = []

	atr = 0
	upper_band = 0
	lower_band = 0
	pecdency = 0

	for i, candle in enumerate(places):
		high = candle['high']
		low = candle['low']
		close = candle['close']

		

		tr = max(high - low, abs(high - close), abs(low - close))

		if i == 0:
			atr = tr
			upper_band = high + multiplier * atr
			lower_band = low - multiplier * atr
		else:
			atr = ((spans - 1) * atr + tr) / spans
			upper_band = high + multiplier * atr
			lower_band = low - multiplier * atr

		pecdency_1 = ((high + low) / 2) + (multiplier * atr)


		if i == 0 or places [i - 1] ['close'] <= pecdency:
			pecdency = upper_band
		else:
			pecdency = lower_band

		atr_values.append(atr)
		upper_band_values.append(upper_band)
		lower_band_values.append(lower_band)
		pecdency_values.append(pecdency)

		candle ['ATR'] = atr
		candle ['pecdency UB'] = upper_band
		candle ['pecdency LB'] = lower_band
		candle ['pecdency'] = pecdency
		candle ['pecdency 1'] = pecdency_1

	return places






'''
	from fortunes.taps.season_1.supertrend.ST_1_ChatGPT import calculate_supertrend
	calculate_supertrend (data, period = 7, multiplier = 3)
'''

'''
Can you provide me with
the supertrend formula in python3 without pandas 
with input as a list of dicts, 
that adds supertrend and atr to the dicts in the input list,
and the input values are "high", "low", and "close",
with the licensing info written as a comment?


Where the license says:
"copyright ChatGPT and OpenAI"
"This work is subject to the terms of a custom license that is a fork of the MIT license."
Where the license is a custom version of the MIT license
that doesn't use any all caps writing?
'''
# Custom License (Fork of MIT License)

# Copyright (c) ChatGPT and OpenAI

# This work is subject to the terms of a custom license that is a fork of the MIT license.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "software"), to deal
# in the software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the software, and to permit persons to whom the software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the software.

# The software is provided "as is," without warranty of any kind, express or
# implied, including but not limited to the warranties of merchantability,
# fitness for a particular purpose, and noninfringement. In no event shall the
# authors or copyright holders be liable for any claim, damages, or other
# liability, whether in an action of contract, tort or otherwise, arising from,
# out of or in connection with the software or the use or other dealings in
# the software.

def calculate_supertrend (data, period=7, multiplier=3):
	tr_list = []
	atr_list = []

	for i in range (1, len (data)):
		TR = max(
			data [i]['high'] - data [i] ['low'],
			abs (data[i]['high'] - data [i - 1]['close']),
			abs (data[i]['low'] - data [i - 1]['close'])
		)

		data [i] ['TR'] = TR

		tr_list.append (TR)

	# Initial ATR value
	atr_list.append(sum(tr_list[:period]) / period)

	for i in range(period, len(tr_list)):
		atr_value = ((period - 1) * atr_list[-1] + tr_list[i]) / period
		atr_list.append(atr_value)

	print ("atr_list:", atr_list)

	for i in range (len (atr_list)):
		supertrend = data[i]['close'] + multiplier * atr_list[i]
		data[i]['ATR'] = atr_list[i]
		data[i]['Supertrend'] = supertrend

# Example usage:
# data = [
#     {'high': 50, 'low': 45, 'close': 48},
#     {'high': 55, 'low': 50, 'close': 52},
#     # ... (more data)
# ]
# calculate_supertrend(data, period=7, multiplier=3)
# print(data)




'''
	import fortunes.taps.season_1.ATR.ChatGPT_ATR_1 as ChatGPT_ATR_1
	ChatGPT_ATR_1.calculate_ATR ()
	
	# Example usage:
	# data = [
	#     {'high': 50, 'low': 45, "close": 48},
	#     {'high': 55, 'low': 50, "close": 52},
	#     # ... (more data)
	# ]
	# calculate_ATR(data, period=14)
	# print(data)
'''

'''
Can you provide me with
the ATR formula in python3 without pandas 
with input as a list of dicts, 
that adds ATR and atr to the dicts in the input list,
and the input values are "high", "low", and "close",
with the licensing info written as a comment?
where the name of the def is calculate_ATR

Where the license says:
"copyright ChatGPT and OpenAI"
"This work is subject to the terms of this custom license that is a fork of the MIT license."
Where the license is a custom version of the MIT license
that doesn't use any all caps writing?

After the license can you say, "These licensing terms were provided as per an ask."

And can you please reference any sources utilized?
'''
# Custom License (Fork of MIT License)

# Copyright (c) ChatGPT and OpenAI

# This work is subject to the terms of this custom license that is a fork of the MIT license.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "software"), to deal
# in the software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the software, and to permit persons to whom the software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the software.

# The software is provided "as is," without warranty of any kind, express or
# implied, including but not limited to the warranties of merchantability,
# fitness for a particular purpose, and noninfringement. In no event shall the
# authors or copyright holders be liable for any claim, damages, or other
# liability, whether in an action of contract, tort or otherwise, arising from,
# out of or in connection with the software or the use or other dealings in
# the software.

# These licensing terms were provided as per an ask.

def calculate_ATR(data, period=14):
    tr_list = []
    atr_list = []

    for i in range(1, len(data)):
        tr = max(
            data[i]['high'] - data[i]['low'],
            abs(data[i]['high'] - data[i - 1]["close"]),
            abs(data[i]['low'] - data[i - 1]["close"])
        )
        tr_list.append(tr)

	# Initial ATR value
	ATR_1 = sum (
		tr_list [:period]
	) / period
	
    atr_list.append (ATR_1)  

    for i in range (period, len (tr_list)):
        atr_value = (
			(period - 1) * atr_list [-1] + tr_list [i]
		) / period
        
		atr_list.append (atr_value)

    for i in range(len(atr_list)):
        data [i] ['ATR'] = atr_list[i]







'''
	import fortunes.clouds.Coinbase.API as Coinbase_API
	proceeds = Coinbase_API.proposal (
		key_name = ellipsis ["name"],
		key_secret = ellipsis ["privateKey"],
		
		request_path = "/api/v3/brokerage/accounts"
	)
	
	from operator import itemgetter
	accounts = proceeds ["accounts"];
	accounts = sorted (accounts, key=itemgetter('name')) 

	for account in accounts:
		print ("name:", account ["name"]);

'''

'''
	https://docs.cloud.coinbase.com/exchange/docs/sandbox
	https://docs.cloud.coinbase.com/advanced-trade-api/reference/retailbrokerageapi_postorder
'''

import rich
import jwt
from cryptography.hazmat.primitives import serialization
import time
import secrets
import http.client
import json
def proposal (
	key_name = "",
	key_secret = "",

	request_method = "GET",
	request_host = "api.coinbase.com",
	request_path = "/api/v3/brokerage/accounts",
	request_body = "",
	request_query_params = "",

	service_name = "retail_rest_api_proxy",
):
	def build_jwt (service, uri):
		private_key_bytes = key_secret.encode('utf-8')
		private_key = serialization.load_pem_private_key(private_key_bytes, password=None)
		jwt_payload = {
			'sub': key_name,
			'iss': "coinbase-cloud",
			'nbf': int(time.time()),
			'exp': int(time.time()) + 60,
			'aud': [service],
			'uri': uri,
		}
		
		#rich.print_json (data = jwt_payload)
		
		jwt_token = jwt.encode(
			jwt_payload,
			private_key,
			algorithm='ES256',
			headers={'kid': key_name, 'nonce': secrets.token_hex()},
		)
		
		#rich.print_json (data = jwt_token)
		
		
		return jwt_token
	
	

	def proposal (
		method = "GET",
		route = "",
		
		payload = "",

		Authorization = ""
	):
		conn = http.client.HTTPSConnection (request_host)
		
		headers = {
			'Content-Type': 'application/json',
			'Authorization': Authorization
		}
		
		#print ("payload:", payload)
		
		conn.request (
			method, 
			route + request_query_params, 
			payload, 
			headers
		)
		
		res = conn.getresponse ()		
		print ("response status:", res.status)
		
		data = res.read ()
		#print ("response data:", data)
		
		
		return {
			"data": data.decode ("utf-8")
		}
	
	
	URI = f"{request_method} {request_host}{request_path}"
	JWT = build_jwt (service_name, URI)
	#print(f"export JWT={JWT}")


	rich.print_json (data = {
		"method": request_method,
		"route": request_path,
		
		"payload": request_body,

		"Authorization": f"Bearer { JWT }"
	})
	
	proceeds = proposal (
		method = request_method,
		route = request_path,
		
		payload = request_body,

		Authorization = f"Bearer { JWT }"
	)
	
	if ("data" not in proceeds):
		print ("proceeds:", proceeds)
	
	#rich.print_json (data = json.loads (proceeds ["data"]))
	
	return json.loads (proceeds ["data"])





'''
	python3 status.py 'stats/aggregate_break_even/_status/status_1.py'
'''

'''
	sources:
		https://www.nasdaq.com/market-activity/stocks/fslr/option-chain
'''


import ships.paths.files.scan.JSON as scan_JSON_path
import fortunes.stats.aggregate_break_even as aggregate_break_even
import fortunes.stats.aggregate_break_even.show as show_aggregate_break_even
	
import pathlib
from os.path import dirname, join, normpath
import sys
this_dir = pathlib.Path (__file__).parent.resolve ()

def check_1 ():
	example = scan_JSON_path.start (normpath (join (this_dir, "examples/1.JSON")))
	proceeds = aggregate_break_even.calc (example)
	
	import json
	print ("proceeds:", json.dumps (proceeds, indent = 4))

	show_aggregate_break_even.data (proceeds)

	
	
checks = {
	"check 1": check_1
}



'''
	import fortunes.stats.aggregate_PC_ratio as aggregate_PC_ratio
	evaluation = aggregate_PC_ratio.calc (example_1)
	
	import json
	print ("evaluation:", json.dumps (evaluation, indent = 4))
'''

'''
	OUTPUT:
	
	{
		"expirations": [{
			"expiration": "2023-10-27",
			"sums": {
				"puts": {
					"ask":
					"bid":
					"last"
				},
				"calls": {
					"ask":
					"bid":
					"last"
				}
			},
			"PC ratios": {
				"ask":
				"bid":
				"last"
			}
		}],
		"sums": {
			"puts": {
				"ask":
				"bid":
				"last"
			},
			"calls": {
				"ask":
				"bid":
				"last"
			}
		},
		"PC ratios": {
			"ask":
			"bid":
			"last"
		}
	}
'''

from fortunes.gadgets.ratio import calculate_ratio

import pydash
def return_number (OBJECT, PATH, DEFAULT):
	FOUND = pydash.get (
		OBJECT,
		PATH,
		DEFAULT
	)
	
	TYPE = type (FOUND)
	if (TYPE == int or TYPE == float):
		return FOUND
		
	if (FOUND == None):
		return DEFAULT;

	print ("FOUND WAS NOT ACCOUNTED FOR:", FOUND)
	raise Exception (f"FOUND WAS NOT ACCOUNTED FOR: { FOUND }")
		
	return DEFAULT

def retrieve_multiplicand (strike):
	return strike ["contract size"] * strike ["open interest"]
 
	try:
		pass;
	
	except Exception as E:
		pass;
		

def EQUALITY_CHECK (PARAM_1, PARAM_2):
	try:
		assert (PARAM_1 == PARAM_2)
	except Exception as E:
		import traceback
		
		print ("PARAM 1", PARAM_1)
		print ("PARAM 2", PARAM_2)	
		
		print (traceback.print_exception (E))

		raise Exception (E)

	return
	

def calc (chain):
	expirations = chain ["expirations"]
	
	evaluation = {
		"expirations": [],
		"sums": {
			"puts": {
				"ask": 0,
				"bid": 0,
				"last": 0
			},
			"calls": {
				"ask": 0,
				"bid": 0,
				"last": 0
			}
		},
		"PC ratios": {
			"ask": 0,
			"bid": 0,
			"last": 0
		}
	}
	
	
	for expiration in expirations:
		calls_strikes = expiration ["calls"]["strikes"]
		puts_strikes = expiration ["puts"]["strikes"]
		
		expiration_note = {
			"expiration": expiration ["expiration"],
			"sums": {
				"puts": {
					"ask": 0,
					"bid": 0,
					"last": 0
				},
				"calls": {
					"ask": 0,
					"bid": 0,
					"last": 0
				}
			},
			"PC ratios": {
				"ask": 0,
				"bid": 0,
				"last": 0
			}
		}
		
		EQUALITY_CHECK (len (calls_strikes), len (puts_strikes))
		
		direction = "calls"
		for strike in calls_strikes:		
			expiration_note ["sums"][ direction ]["ask"] += (
				return_number (strike, [ "prices", "ask" ], 0) * 
				retrieve_multiplicand (strike)
			)
			expiration_note ["sums"][ direction ]["bid"] += (
				return_number (strike, [ "prices", "bid" ], 0) * 
				retrieve_multiplicand (strike)
			)
			expiration_note ["sums"][ direction ]["last"] += (
				return_number (strike, [ "prices", "last" ], 0) * 
				retrieve_multiplicand (strike)
			)
		
		direction = "puts"
		for strike in puts_strikes:		
			expiration_note ["sums"][ direction ]["ask"] += (
				return_number (strike, [ "prices", "ask" ], 0) * 
				retrieve_multiplicand (strike)
			)
			expiration_note ["sums"][ direction ]["bid"] += (
				return_number (strike, [ "prices", "bid" ], 0) * 
				retrieve_multiplicand (strike)
			)
			expiration_note ["sums"][ direction ]["last"] += (
				return_number (strike, [ "prices", "last" ], 0) * 
				retrieve_multiplicand (strike)
			)
		
		expiration_note ["PC ratios"]["ask"] = calculate_ratio (
			expiration_note ["sums"][ "puts" ]["ask"],
			expiration_note ["sums"][ "calls" ]["ask"]
		)
		expiration_note ["PC ratios"]["bid"] = calculate_ratio (
			expiration_note ["sums"][ "puts" ]["bid"],
			expiration_note ["sums"][ "calls" ]["bid"]
		)
		expiration_note ["PC ratios"]["last"] = calculate_ratio (
			expiration_note ["sums"][ "puts" ]["last"],
			expiration_note ["sums"][ "calls" ]["last"]
		)
		
		evaluation ["sums"][ "calls" ]["ask"] += return_number (expiration_note, [ "sums", "calls", "ask" ], 0)
		evaluation ["sums"][ "calls" ]["bid"] += return_number (expiration_note, [ "sums", "calls", "bid" ], 0)
		evaluation ["sums"][ "calls" ]["last"] += return_number (expiration_note, [ "sums", "calls", "last" ], 0)
		
		evaluation ["sums"][ "puts" ]["ask"] += return_number (expiration_note, [ "sums", "puts", "ask" ], 0)
		evaluation ["sums"][ "puts" ]["bid"] += return_number (expiration_note, [ "sums", "puts", "bid" ], 0)
		evaluation ["sums"][ "puts" ]["last"] += return_number (expiration_note, [ "sums", "puts", "last" ], 0)
		
		evaluation ["expirations"].append (expiration_note)
		
	evaluation ["PC ratios"]["ask"] = calculate_ratio (
		evaluation ["sums"][ "puts" ]["ask"],
		evaluation ["sums"][ "calls" ]["ask"]
	)
	
	evaluation ["PC ratios"]["bid"] = calculate_ratio (
		evaluation ["sums"][ "puts" ]["bid"],
		evaluation ["sums"][ "calls" ]["bid"]
	)
	
	evaluation ["PC ratios"]["last"] = calculate_ratio (
		evaluation ["sums"][ "puts" ]["last"],
		evaluation ["sums"][ "calls" ]["last"]
	)

	return evaluation






'''
	AAS: Average True Range
'''

'''
	import fortunes.taps.season_1.AAS as AAS_indicator
	
	AAS = AAS_indicator.calc (
		data = [{
            "high": 100,
            "low": 80,
            "close": 99
		}],
		
		spans = 7
	)
'''

'''
	AAS = (1 / spans) * summation (TR, 1, spans)
'''

import fortunes.taps.season_2.AS as AS_tap

def summation_TRs (
	places = []
):
	sum = 0
	for place in places:
		sum += place ["AS"]
		
	return sum;
		
def calc_AAS_1 (
	places = [],
	spans = None
):
	return summation_TRs (places) / spans
	
def calc (
	places = [],
	
	label = "AAS",
	
	#
	#	The number of places to consider in the
	#	first calculation.
	#
	spans = 7
):
	AS_tap.calc (
		places = places,
		label = "AS"
	)

	'''
	
	'''
	assert (
		len (places) >= spans + 1
	), [ len (places), spans ]


	'''
	
	'''
	span_1 = places [1:] [:spans];
	assert (len (span_1) >= spans)
	
	
	AAS_1 = calc_AAS_1 (
		places = places [1:] [:spans],
		spans = spans
	)
	
	places [spans] [label] = AAS_1


	AAS = AAS_1
	
	last_places_index = len (places) - 1;
	s = spans + 1
	while (s <= last_places_index):
		AAS = (
			(AAS * (spans - 1)) + 
			places [s] ["AS"]
		) / spans;
		
		places [s] [label] = AAS
	
	
		s += 1;





'''
	import fortunes.gadgets.pandas.DF.from_list as df_from_list
	list = list_from_df.calc (
		df = df
	)
'''

'''
import pandas
DF = pandas.DataFrame (list)
'''

import pandas

def calc (
	list = None
):
	DF = pandas.DataFrame (list)

	return DF




'''
	https://stackoverflow.com/questions/29815129/pandas-dataframe-to-list-of-dictionaries
'''
'''
	import pandas
	catalogue_of_objects = DF.to_dict ('records')
'''

import pandas

def calc (
	df = None
):
	return df.to_dict ('records')
	




'''
	import ccxt
	import fortunes.clouds.CCXT.symbols.array as CCXT_symbols

	symbols = CCXT_symbols.retrieve (
		exchange = ccxt.kraken ()
	)
'''


def retrieve (
	exchange = ""
):
	symbols = exchange.symbols;
	markets = exchange.load_markets ()
	symbols_list = []
	for market in markets:
		symbols_list.append (markets [market] ["id"])
		
	return symbol



'''
	python3 insurance.proc.py taps/season_2/PS/_status/status_1.py
'''

import fortunes.taps.season_3.physical_span as PS_tap

from rich import print_json

def check_1 ():	
	places = [{
		"high": 38404.3875,
		"low": 36901.035,
		"close": 38056.5515
	},{
		"high": 38423.07,
		"low": 37602.688,
		"close": 37895.439
	}]
	
	PS_tap.calc (
		#
		#	data 
		#
		places = places,
		label = "PS"
	)
	
	print_json (data = places)
	
	
	assert (places [1] ["PS"] == 820.3819999999978)
	
	
	
checks = {
	'check 1': check_1
}







'''
import fortunes.climate as climate
climate.change ("ports", {
	"driver": 18871,
	"cluster": 0,
	"http": 0	
})
'''

'''
import fortunes.climate as climate
Tradier = climate.find ("Tradier")
'''

import copy

def retrieve_Tradier ():
	import json
	fp = open ("/online ellipsis/tradier.com/ellipse.json", "r")
	Tradier_authorization = json.loads (fp.read ()) ["API"]
	fp.close ()
	
	return Tradier_authorization
	

climate = {
	"Tradier": {
		"authorization": retrieve_Tradier ()
	}
}

def change (field, plant):
	#global CLIMATE;
	climate [ field ] = plant


def find (field):
	return copy.deepcopy (climate) [ field ]




'''

'''

import backtrader as bt
import backtrader.feeds as btfeeds
from datetime import datetime, timedelta
import yfinance as yf

class Strategy_1 (bt.Strategy):
	def __init__(self):
		# Keep a reference to the "close" line in the data[0] dataseries
		self.dataclose = self.datas[0].close

		# To keep track of pending orders
		self.order = None
		
		#bt.indicators.ExponentialMovingAverage(self.datas[0], period=25)
		#bt.indicators.WeightedMovingAverage(self.datas[0], period=25).subplot = True
		#bt.indicators.StochasticSlow(self.datas[0])
		#bt.indicators.MACDHisto(self.datas[0])
		#rsi = bt.indicators.RSI(self.datas[0])
		#bt.indicators.SmoothedMovingAverage(rsi, period=10)
		#bt.indicators.ATR(self.datas[0]).plot = False

	'''
	
	'''
	def log (self, txt, dt=None):
		''' Logging function fot this strategy'''
		dt = dt or self.datas[0].datetime.date(0)
		print('%s, %s' % (dt.isoformat(), txt))

	

	def notify_order (self, order):
		if order.status in [ order.Submitted, order.Accepted ]:
			# Buy/Sell order submitted/accepted to/by broker - Nothing to do
			return

		# Check if an order has been completed
		# Attention: broker could reject order if not enough cash
		if order.status in [ order.Completed ]:
			if order.isbuy ():
				self.log ('pump occurred, %.2f' % order.executed.price)
			
			elif order.issell ():
				self.log ('dump occurred, %.2f' % order.executed.price)

			self.bar_executed = len(self)

		elif order.status in [order.Canceled, order.Margin, order.Rejected]:
			self.log('Order Canceled/Margin/Rejected')

		# Write down: no pending order
		self.order = None


	'''
	
	'''
	def next (self):
		# Simply log the closing price of the series from the reference
		self.log ('Close, %.2f' % self.dataclose[0])

		
		'''
			if an order is pending, a second one cannot be sent
		'''
		if self.order:
			return

		# Check if we are in the market
		if not self.position:

			# Not yet ... we MIGHT BUY if ...
			if self.dataclose[0] < self.dataclose[-1]:
				# current close less than previous close

				if self.dataclose[-1] < self.dataclose[-2]:
					# previous close less than the previous close

					# BUY, BUY, BUY!!! (with default parameters)
					self.log ('creating a pump, %.2f' % self.dataclose[0])

					# Keep track of the created order to avoid a 2nd order
					self.order = self.buy ()

		else:
			# Already in the market ... we might sell
			if len (self) >= (self.bar_executed + 5):
				self.log ('creating a dump, %.2f' % self.dataclose[0])

				# Keep track of the created order to avoid a 2nd order
				self.order = self.sell ()










'''
	PSA: Average True Range
'''

'''
	import fortunes.taps.season_3.physical_span_average as PSA_taps
	PSA = PSA_taps.calc (
		places = ,
		spans = 7
	)
'''

'''
	PSA = (1 / spans) * summation (TR, 1, spans)
'''

import fortunes.taps.season_3.physical_span as PS_taps

def summation_PSs (
	places = []
):
	sum = 0
	for place in places:
		sum += place ["PS"]
		
	return sum;

def calc_PSA_1 (
	places = [],
	spans = None
):
	return summation_PSs (places) / spans
	
def calc (
	places = [],
	
	label = "PSA",
	
	#
	#	The number of places to consider in the
	#	first calculation.
	#
	spans = 7
):
	assert (len (places) >= spans + 1), [ len (places), spans ]
	
	PS_taps.calc (
		places = places,
		label = "PS"
	)


	'''
	
	'''
	span_1 = places [1:] [:spans];
	assert (len (span_1) >= spans)
	
	#PSA_1 = summation_PSs (places) / spans
	
	PSA_1 = calc_PSA_1 (
		places = places [1:] [:spans],
		spans = spans
	)
	
	
	places [spans] [label] = PSA_1

	PSA = PSA_1
	

	
	last_places_index = len (places) - 1;
	s = spans + 1
	while (s <= last_places_index):
		PSA = (
			(PSA * (spans - 1)) + 
			places [s] ["PS"]
		) / spans;
		
		places [s] [label] = PSA
	
		s += 1;











'''
	from fortunes.tools.ratio import calculate_ratio
'''

def calculate_ratio (s1, s2):
	if (s1 == 0 or s2 == 0):
		return [ "?", "?" ]
		#return "can't divide by zero"
		
		#return [ "~ >= infinity", 0 ]
		#return [ "infinity", 1 ]

	if (s1 == s2):
		return [ 1, 1 ]
	
	elif (s1 >= s2):
		return [
			s1 / s2,
			1
		]
	
	elif (s2 >= s1):
		return [
			1,
			s2 / s1
		]
	
	
	raise ("?")




'''
	import fortunes.taps.season_1.supertrend as supertrend_indicator
	supertrend_indicator.calc (
		places = [],
		spans = 4
	)
'''

import fortunes.taps.season_1.ATR as ATR_indicator

def calc (
	places = [],
	spans = None,
	multiplier = 1.4
):
	ATR_indicator.calc (
		places = places,
		label = "ATR",
		
		spans = spans
	)
	
	for i in range (spans, len (places)):
		if ("ATR" not in places [i]):
			continue;
			
		place_1 = places [i - 1]
		place_2 = places [i]
	
		upper_band = place_1 ['close'] + multiplier * place_2 ['ATR']
		lower_band = place_1 ['close'] - multiplier * place_2 ['ATR']
		end = place_1 ['close']

		if end > upper_band:
			place_2 ['supertrend'] = lower_band
			place_2 ['position'] = 1  # Buy Signal
			
			print ("Buy")
			
		elif end < lower_band:
			place_2 ['supertrend'] = upper_band
			place_2 ['position'] = -1  # Sell Signal

			print ("Sell")

		else:
			print ("in between bands")
		
			try:
				place_2 ['supertrend'] = place_1 ['supertrend']
				place_2 ['position'] = 0
			except Exception:
				#print ('exception:', Exception)
				pass;
			
		



'''
	python3 insurance.proc.py 'gadgets/pandas/df/_status/status_from_1.py'
'''

import fortunes.gadgets.pandas.DF.from_list as df_from_list
import fortunes.gadgets.pandas.DF.to_list as df_to_list

def check_1 ():	
	original_list =  [{
		"open": 10,
		"close": 11,
		"high": 20,
		"low": 9
	}]

	df = df_from_list.calc (
		list = [{
			"open": 10,
			"close": 11,
			"high": 20,
			"low": 9
		}]
	)
	
	steps = df_to_list.calc (df)
		
	assert (
		original_list ==
		steps
	)
	
checks = {
	'check 1': check_1
}






'''
	from operator import itemgetter
	import fortunes.clouds.Coinbase.API.products.catalogue as Coinbase_API_products_catalogue
	products = Coinbase_API_products_catalogue.proposal (
		key_name = ellipsis ["name"],
		key_secret = ellipsis ["privateKey"]
	)

	product_IDs = []
	for product in products:
		if (not product ["product_id"].endswith ("-USD")):
			continue;

		product_IDs.append (product ["product_id"])
		continue;

		#print ("product_id:", product ["product_id"]);
		rich.print_json (data = product)
	#

	print (product_IDs)
'''


import fortunes.clouds.Coinbase.API as Coinbase_API
from operator import itemgetter

def proposal (
	key_name = "",
	key_secret = "",
	
	request_path = "/api/v3/brokerage/products"
):
	proceeds = Coinbase_API.proposal (
		key_name = key_name,
		key_secret = key_secret,
		
		request_path = request_path
	)

	products = proceeds ["products"];
	products = sorted (products, key = itemgetter ('name')) 
		
	return products;